@@ -19,11 +19,12 @@
 use super::*;
 use crate::decoder::tiling::{self, FrameBufferExt};
 use crate::deps::*;
+use crate::image_metadata::ImageMetadata;
 use crate::util;
 use crate::util::gettext::*;
 use crate::util::ToBufRead;
 
-use anyhow::Context;
+use anyhow::{anyhow, bail, Context};
 use arc_swap::ArcSwap;
 use gtk::prelude::*;
 use libheif_rs::{ColorProfile, ColorSpace, HeifContext, LibHeif, Plane, RgbChroma};
@@ -33,7 +34,15 @@ use rgb::AsPixels;
 use std::sync::Arc;
 
 #[derive(Debug)]
-pub struct Heif;
+pub struct Heif {
+    cancellable: gio::Cancellable,
+}
+
+impl Drop for Heif {
+    fn drop(&mut self) {
+        self.cancellable.cancel();
+    }
+}
 
 /// Loads plugins on first use
 static LIBHEIF: Lazy<LibHeif> = Lazy::new(LibHeif::new);
@@ -45,7 +54,10 @@ impl Heif {
         tiles: Arc<ArcSwap<tiling::FrameBuffer>>,
     ) -> Self {
         log::debug!("Loading HEIF");
-        updater.spawn_error_handled(move || {
+        let cancellable = gio::Cancellable::new();
+        let cancellable_ = cancellable.clone();
+
+        updater.clone().spawn_error_handled(move || {
             let file_size = file
                 .query_info(
                     gio::FILE_ATTRIBUTE_STANDARD_SIZE,
@@ -54,12 +66,19 @@ impl Heif {
                 )
                 .context(gettext("Failed to read image file information"))?
                 .size();
-            let buf_reader = file.to_buf_read()?;
+            let buf_reader = file.to_buf_read(&cancellable)?;
             let stream_reader = libheif_rs::StreamReader::new(buf_reader, file_size as u64);
             let ctx = HeifContext::read_from_reader(Box::new(stream_reader))
                 .context(gettext("Failed to decode image"))?;
             let handle = ctx.primary_image_handle()?;
 
+            let mut metadata = Self::exif(&handle, &file);
+
+            // TODO: Later use libheif 1.16 to get info if there is a transformation
+            metadata.heif_transform = true;
+
+            updater.send(DecoderUpdate::Metadata(metadata));
+
             tiles.set_original_dimensions((handle.width(), handle.height()));
 
             let rgb_chroma = if handle.luma_bits_per_pixel() > 8 {
@@ -88,9 +107,17 @@ impl Heif {
                 RgbChroma::Rgb
             };
 
-            let mut image = LIBHEIF
-                .decode(&handle, ColorSpace::Rgb(rgb_chroma), None)
-                .context(gettext("Failed to decode image"))?;
+            let image_result = LIBHEIF.decode(&handle, ColorSpace::Rgb(rgb_chroma), None);
+
+            let mut image = match image_result {
+                Err(err)
+                    if matches!(err.sub_code, libheif_rs::HeifErrorSubCode::UnsupportedCodec) =>
+                {
+                    updater.send(DecoderUpdate::UnsupportedFormat);
+                    bail!(anyhow!(err).context(gettext("Unknown image format")));
+                }
+                image => image.context(gettext("Failed to decode image"))?,
+            };
 
             let icc_profile = if let Some(profile) = handle.color_profile_raw() {
                 if [
@@ -132,7 +159,39 @@ impl Heif {
             Ok(())
         });
 
-        Heif
+        Heif {
+            cancellable: cancellable_,
+        }
+    }
+
+    fn exif(handle: &libheif_rs::ImageHandle, file: &gio::File) -> ImageMetadata {
+        let mut meta_ids = vec![0];
+        handle.metadata_block_ids(&mut meta_ids, b"Exif");
+
+        if let Some(meta_id) = meta_ids.first() {
+            match handle.metadata(*meta_id) {
+                Ok(mut exif_bytes) => {
+                    if let Some(skip) = exif_bytes
+                        .get(0..4)
+                        .map(|x| u32::from_be_bytes(x.try_into().unwrap()) as usize)
+                    {
+                        if exif_bytes.len() > skip + 4 {
+                            exif_bytes.drain(0..skip + 4);
+                            return ImageMetadata::from_exif_bytes(exif_bytes);
+                        } else {
+                            log::warn!("EXIF data has far too few bytes in {}", file.uri());
+                        }
+                    } else {
+                        log::warn!("EXIF data has far too few bytes in {}", file.uri());
+                    }
+                }
+                Err(err) => {
+                    log::warn!("Unable to decode EXIF data for {}: {err}", file.uri());
+                }
+            }
+        }
+
+        ImageMetadata::default()
     }
 }
 
