@@ -176,6 +176,7 @@ mod imp {
                         gesture.set_state(gtk::EventSequenceState::Denied);
                         None
                     } else {
+                        gesture.set_state(gtk::EventSequenceState::Claimed);
                         obj.current_page().and_then(|p| p.content_provider())
                     }
                 }),
@@ -209,8 +210,14 @@ mod imp {
             let (overlay1_min, _, _, _) = self.controls_box_start.measure(orientation, for_size);
             let (overlay2_min, _, _, _) = self.controls_box_end.measure(orientation, for_size);
 
+            let overlay_min = match orientation {
+                gtk::Orientation::Horizontal => overlay1_min + overlay2_min,
+                gtk::Orientation::Vertical => i32::max(overlay1_min, overlay2_min),
+                _ => unreachable!(),
+            };
+
             // take as minimum whatever is larger
-            let min = i32::max(child_min, overlay1_min + overlay2_min);
+            let min = i32::max(child_min, overlay_min);
             // take as natural whatever is larger
             let natural = i32::max(child_natural, min);
 
@@ -256,7 +263,7 @@ impl LpImageView {
         if let Some(first) = files.first().cloned() {
             let model = LpFileModel::from_files(files);
             self.set_model(model);
-            let page = LpImagePage::from_file(&first);
+            let page = self.new_image_page(&first);
 
             sliding_view.clear();
             sliding_view.append(&page);
@@ -356,7 +363,7 @@ impl LpImageView {
         let page = if let Some(page) = sliding_view.get(new_file) {
             page
         } else {
-            let page = LpImagePage::from_file(new_file);
+            let page = self.new_image_page(new_file);
 
             if new_index > current_index {
                 sliding_view.append(&page);
@@ -397,7 +404,7 @@ impl LpImageView {
             if let Some(page) = existing.get(&file.uri()) {
                 sliding_view.move_to(page, position);
             } else {
-                sliding_view.insert(&LpImagePage::from_file(file), position);
+                sliding_view.insert(&self.new_image_page(file), position);
             }
         }
 
@@ -438,6 +445,32 @@ impl LpImageView {
         self.imp().model.borrow().clone()
     }
 
+    /// Create new image that communicates updates to file model
+    fn new_image_page(&self, file: &gio::File) -> LpImagePage {
+        let page = LpImagePage::from_file(file);
+
+        page.image().connect_notify_local(
+            Some("is-unsupported"),
+            glib::clone!(@weak self as obj => move |image, _| {
+                if image.is_unsupported() {
+                    if obj.current_image().as_ref() == Some(image) {
+                        log::debug!(
+                            "Image format unsupported but not removing since current image"
+                        );
+                        return;
+                    }
+
+                    if let Some(file) = image.file() {
+                        log::debug!("Removing image with unsupported format {:?}", file.uri());
+                        obj.model().remove(&file);
+                    }
+                }
+            }),
+        );
+
+        page
+    }
+
     fn set_model(&self, model: LpFileModel) {
         model.connect_changed(
             glib::clone!(@weak self as obj => move || obj.model_content_changed_cb()),
