@@ -1,6 +1,5 @@
-// mod.rs
-//
-// Copyright 2020 Christopher Davis <christopherdavis@gnome.org>
+// Copyright (c) 2020-2022 Christopher Davis
+// Copyright (c) 2023 Sophie Herold
 //
 // This program is free software: you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -20,11 +19,15 @@
 mod image;
 mod image_page;
 mod image_view;
+mod print;
+mod print_preview;
 mod properties_view;
 mod sliding_view;
 
 pub use image::LpImage;
 pub use image_page::LpImagePage;
 pub use image_view::LpImageView;
+pub use print::LpPrint;
+pub use print_preview::LpPrintPreview;
 pub use properties_view::LpPropertiesView;
 pub use sliding_view::LpSlidingView;
